<?php
/**
 * NLWISZoomBar Widget class
*/
include_once($_SESSION['gszCorePath']."widgets/ScaleZoom/ScaleZoom.widget.php");

/**
 * NLWISZoomBar
 *
 * @desc Display scale bar that the user can zoom to a different.
 * scale using.
 */
class NLWISZoomBar extends ScaleZoom
{
    var $nMaxWidthFactor = "";
    var $nLevels = 0;
    /**
     * NLWISZoomBar
     *
     * Constructor for the NLWISZoomBar
     */
    function NLWISZoomBar( )
    {
        // invoke constructor of parent
        parent::NavTool();

        $this->SetNavCommand("SCALE_ZOOM");

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The NLWISZoomBar widget displays a series ticks the user can zoom in with.
EOT;

        $this->maAttributes["TOPELEMENT"] = new StringAttribute("TOPELEMENT", false);
        $this->maAttributes["ZOOMINLABEL"] = new StringAttribute("ZOOMINLABEL", false);
        $this->maAttributes["ZOOMOUTLABEL"] = new StringAttribute("ZOOMOUTLABEL", false);
        $this->maAttributes["LEVELS"] = new StringAttribute("LEVELS", false);

        $this->mnMaturityLevel = MATURITY_BETA;
    }

    function InitDefaults()
    {
        parent::InitDefaults();

        /* Get priority for levels
         * 1- mapfile - "scale_zoombar" metadata at map object
         * 2- widget definition in template
         * 3- otherwise - 5
        */
        if ($this->moMapObject->oMap != null && $this->moMapObject->oMap->getMetaData("zoombar_levels") != "") {
            $this->nLevels = $this->moMapObject->oMap->getMetadata("zoombar_levels");
        } elseif (isset($this->maParams["LEVELS"])) {
            $this->nLevels = (int)$this->maParams["LEVELS"];
        } else {
            $this->nLevels = 5;
        }

    }
    function GetJavascriptVariables()
    {
        // init vars
        $aReturn = parent::GetJavascriptVariables();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"])) {
            if (isset($this->nLevels) && $this->nLevels > 0) {
                $nLevels =  $this->nLevels;
            } else {
                $nLevels = 5;
            }

            $nMinScale = $this->getMinScale();
            $nMaxScale = $this->getMaxScale();

            $aScale = $this->getZoomBarLevels($nLevels,$nMinScale,$nMaxScale);
            $strLevels = implode(",",$aScale);
            $aReturn['aNlwisZoomBarLevels'] = "var aNlwisZoomBarLevels = [".$strLevels."];"."\n";
            $aReturn['nNlwisZoomBarCurrentScale'] = "var nNlwisZoomBarCurrentScale = ".(int)$this->moMapObject->oMap->scale.";"."\n";
            $aReturn['oNlwisZoomBarParentElementId'] = "var oNlwisZoomBarParentElementId = '".$this->maParams['TOPELEMENT']."'";
        }

        return $aReturn;

    // end GetJavascriptVariables() function
    }

    function GetJavascriptIncludeFunctions()
    {
        $aReturn = parent::GetJavascriptIncludeFunctions();
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"])) {
            $szVar = "NLWISZoomBar_jsapi.js";
            $aReturn[$szVar] = "<script src=\"".$_SESSION['gszCoreWebPath'].
                                "/widgets/NLWISZoomBar/$szVar\" type=\"text/javascript\"></script>";
        }
        return $aReturn;

    }

    /**
     * GetJavascriptFunctions
     *
     * Build and return the array of functions needed in the
     * widget.
     */
    function GetJavascriptFunctions()
    {
        $aReturn = parent::GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        $szJsFunctionName = "clickScaleValue";
        $szFunction = <<<EOT

function {$szJsFunctionName}(szScale)
{
    if (szScale == null) {
        return;
    }
    {$this->mszHTMLForm}.NAV_CMD.value = "SCALE_ZOOM";
    {$this->mszHTMLForm}.SCALE_ZOOM.value = szScale;

    if ($bCWCJSAPI)
    {
        goCWCJSAPI.SCALE_ZOOM = szScale;
        NLWISZoomBarWUpdate();
    }
    else
    {
        {$this->mszHTMLForm}.submit();
    }

    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

if ($bCWCJSAPI) {
        $szJsFunctionName = "NLWISZoomBarWUpdate";
        $szFunction = <<<EOT

function {$szJsFunctionName}(){
        goCWCJSAPI.SCALE_MAXWIDTHFACTOR = {$this->mszHTMLForm}.SCALE_MAXWIDTHFACTOR.value;
        NLWISZoomBarWUpdateScale({$this->nLevels});}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;


        $szJsFunctionName = "NLWISZoomBarWRegisterForEvent";
        $szFunction = <<<EOT

function {$szJsFunctionName}(){
    goCWCJSAPI.RegisterEvent(MAP_EXTENT_CHANGED, "NLWISZoomBarWUpdateDisplay");
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

} //end function if jsapi

        return $aReturn;
    }

    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = array();

        $aReturn = parent::GetJavascriptOnLoadFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
        {
             $szJsFunctionName = "NLWISZoomBarWRegisterForEvent";
             $szFunction = "$szJsFunctionName();\n";
             $aReturn[$szJsFunctionName] = $szFunction;
        }

        return $aReturn;
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = parent::GetHTMLHiddenVariables();

        $szVariable = "SCALE_MAXWIDTHFACTOR";
        if (isset($this->nMaxWidthFactor) && $this->nMaxWidthFactor > 0) {
            $szValue = $this->nMaxWidthFactor;
        } else {
            $szValue = $this->setMaxWidthFactor();
        }
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=".$szValue.">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * DrawPublish
     *
     * Return the HTML code using the name in the map file and the
     * parameters of the CWC tag.
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- NLWISZoomBar widget hidden -->";

        $szResult = "";
        $szJsOnClick = "";

        if (!isset($this->nMaxWidthFactor) || $this->nMaxWidthFactor == "") {
            $this->setMaxWidthFactor();
        }

        $nCurrentScale = (int)$this->moMapObject->oMap->scale;

        $nMaxScale = $this->getMaxScale();
        $nMinScale = $this->getMinScale();
        $aScale = $this->getZoomBarLevels($this->nLevels,$nMinScale,$nMaxScale);

        $nIncreaseTo = (int)($nMaxScale); //set properly later on
        $nDecreaseTo = (int)($nMinScale);

        $nCurrentZoomLevel = 0; //index of current scale
        $nCurrentScaleDiff = $nMaxScale - $nMinScale;

        for($i=0;$i < count($aScale);$i++)
        {
            if(abs($nCurrentScale - $aScale[$i]) < $nCurrentScaleDiff)
            {
                $nCurrentScaleDiff = abs($nCurrentScale - $aScale[$i]);
                $nCurrentZoomLevel = $i; //set index of current zoom value

                if(isset($aScale[$i+1])) //set scale value of [+] button
                    $nIncreaseTo = $aScale[$i+1];
                else
                    $nIncreaseTo = $aScale[count($aScale)-1];

                if(isset($aScale[$i-1])) //set scale value of [-] button
                    $nDecreaseTo = $aScale[$i-1];
                else
                    $nDecreaseTo = $aScale[0];
            }
        }

        $onClick = "clickScaleValue";
        $szResult .= "\n<div id=\"NLWISZoomBarDiv\">\n";
        $szResult .= "\t<ul class=\"zoomControl\">\n";
        $szResult .= "\t\t<li class=\"caption\">".$this->maParams["ZOOMOUTLABEL"]."</li>\n";

        if ($nDecreaseTo != "-1")
        {
            $szStart = "<a class=\"zoomPlusMinus\" href=\"javascript: clickScaleValue($nDecreaseTo);\">";
            $szEnd = "</a>";
        }
        $szResult .= "<li class=\"zoomlevel\">$szStart-$szEnd</li>\n";

        for ($i=0;$i < count($aScale);$i++)
        {
            $szScale = $aScale[$i];

            if ($szScale == "-1") {
                $szResult .= "<li class=\"zoomlevel\"><a class=\"zoomBtn\">&nbsp;</a></li>";
                $nCurrentZoomLevel++;
            } else if($i == $nCurrentZoomLevel) {
                $szResult .= "<li class=\"zoomlevel\"><a class=\"zoomBtnActive\">&nbsp;</li>";
            } else {
                $szResult .= "<li class=\"zoomlevel\"><a class=\"zoomBtn\" href=\"javascript: clickScaleValue($szScale);\" alt=\"$szScale\">&nbsp;</a></li>";
            }
        }

        $szStart = "<a class=\"zoomPlusMinus\" href=\"javascript: clickScaleValue($nIncreaseTo)\">";
        $szEnd = "</a>";
        $szResult .= "<li class=\"zoomlevel\">$szStart+$szEnd</li>\n";

        $szResult .= "\t\t<li class=\"caption\">".$this->maParams["ZOOMINLABEL"]."</li>\n";
        $szResult .= "\t</ul>\n";
        $szResult .= "</div>\n";


        return $szResult;

    }
    function getMaxScale() {
        $poMap = $this->moMapObject->oMap;
        if (!isset($this->nMaxWidthFactor) || $this->nMaxWidthFactor == "") {
            $this->setMaxWidthFactor();
        }
        $nMaxScale = $this->nMaxWidthFactor/$poMap->width;
        return $nMaxScale;
    }
    function getMinScale() {
        $poMap = $this->moMapObject->oMap;
        if ($poMap->web->minscale != -1) {
            $nMinScale = $poMap->web->minscale; //not adjusted
        } else {
            $nMinScale = 1;
        }
        return $nMinScale;
    }
    function setMaxWidthFactor() {
        $poMap = $this->moMapObject->oMap;

        if (!isset($this->nMaxWidthFactor) || $this->nMaxWidthFactor == ""){
            if ($this->isVarSet("SCALE_MAXWIDTHFACTOR") && $this->getVar("SCALE_MAXWIDTHFACTOR") != ""){
                $this->nMaxWidthFactor = $this->getVar("SCALE_MAXWIDTHFACTOR");
            } else {
                if ($poMap->web->maxscale > 0 && $poMap->web->maxscale > $poMap->scale) {
                    $maxRef = $poMap->web->maxscale;
                } else {
                    $maxRef = $poMap->scale;
                }
                $this->nMaxWidthFactor = $maxRef*$poMap->width;
            }
        }
        return $this->nMaxWidthFactor;
    }
    function getZoomBarLevels($nLevels,$nMinScale,$nMaxScale) {

        $aScale = array();
        $nScalePer = exp(log($nMaxScale/$nMinScale)/($nLevels-1));

        array_push( $aScale, (int)$nMaxScale );
        $lastScale = $nMaxScale;
        for ($i=1;$i < $nLevels;$i++) //starts at 1 because maxscale is index 0
        {
            if ($i == ($nLevels - 1)) {
                $nScale = $nMinScale; //add minscale explicity - math isn't working after rounding error
            } else {
                $nScale = (int)($lastScale/$nScalePer);
            }
            $lastScale = $nScale;
            if ($nScale != "-1")
            {
                array_push( $aScale, $nScale );
            }
        }
        rsort($aScale);

        return $aScale;
    }

}
?>
